<?php 

// Zjednodušená implementace rozhraní NodeIterator z DOM Traversal

// Konstanty zastupující jednotlivé typy uzlů
define("XML_SHOW_ALL", 0xFFFFFFFF);
define("XML_SHOW_ELEMENT", 0x00000001);
define("XML_SHOW_ATTRIBUTE", 0x00000002);
define("XML_SHOW_TEXT", 0x00000004);
define("XML_SHOW_CDATA_SECTION", 0x00000008);
define("XML_SHOW_ENTITY_REFERENCE", 0x00000010);
define("XML_SHOW_ENTITY", 0x00000020);
define("XML_SHOW_PROCESSING_INSTRUCTION", 0x00000040);
define("XML_SHOW_COMMENT", 0x00000080);
define("XML_SHOW_DOCUMENT", 0x00000100);
define("XML_SHOW_DOCUMENT_TYPE", 0x00000200);
define("XML_SHOW_DOCUMENT_FRAGMENT", 0x00000400);
define("XML_SHOW_NOTATION", 0x00000800);

// Třída implementující DOM NodeIterator a zároveň rozhraní Iterator z PHP
class NodeIterator implements Iterator
{
  // pomocné proměnné
  private $root = null;
  private $currentNode = null;
  private $whatToShow = XML_SHOW_ALL;
  private $firstItem = true;

  // konstruktor
  function NodeIterator($root, $whatToShow)
  {
    $this->root = $root;
    $this->currentNode = $root;
    $this->whatToShow = $whatToShow;
  }

  // přesun na další uzel a jeho vrácení
  function nextNode()
  {
    $accepted = false;
    $node = $this->currentNode;

    // prohledávají se další uzly, dokud se nenajde ten se správným typem
    while (!$accepted && $node != null)
    {
      $node = $this->anyNextNode($node);
      if (($node != null) && ((1 << ($node->nodeType - 1)) & $this->whatToShow)) 
      {
        $accepted = true;
      }
    }

    // zapamatování nalezeného uzlu a jeho vrácení
    $this->currentNode = $node;
    return $this->currentNode;
  }

  // vrátí další uzel bez ohledu na jeho typ
  private function anyNextNode($node)
  {
    // neexistující uzel nemá další uzel
    if ($node == null) return null;

    // nejprve zkusíme podelementy a zkusíme první
    if ($node->hasChildNodes()) 
    {
      $node = $node->firstChild;
      return $node;
    }

    // pokud nemá uzel děti, zkusíme nejbližšího dalšího sourozence
    if ($node->nextSibling != null)
    {
      $node = $node->nextSibling;
      return $node;
    }
       
    // hledáme nejbližšího rodiče, který má následujícího sourozence,
    // ale nedostaneme se výše, než je uzel, pro který byl NodeIterator vytvořen
    $node = $node->parentNode;
    while (($node != null) && ($node !== $this->root))
    {
      if ($node->nextSibling != null)
      {
        $node = $node->nextSibling;
        return $node;
      }
      else
      {
        $node = $node->parentNode;
      }
    }
        
    // když další uzel nenajdeme, vrátíme null
    return null;            
  }

  // následující metody implementují rozhraní Iterator, 
  // aby bylo možné instanci této třídy použít přímo ve foreach
  function rewind()
  {
    $this->currentNode = $this->root;
    $this->firstItem = true;
  }

  function current()
  {
    if ($this->firstItem)
    {
      $this->nextNode();
      $this->firstItem = false;
    }
    return $this->currentNode;
  }
 
  function key()
  {
    return null;
  }
 
  function next()
  {
    $this->nextNode();    
  }
  
  function valid()
  {
    return $this->currentNode != null;
  }

}

?>