<?php
Header("Content-type: text/html; charset=utf-8");

// jméno zpracovávaného souboru
$filename  = "../data/faktura.xml";     

// vytvoření parseru
$parser = xml_parser_create_ns();

// nastavení parametrů
xml_parser_set_option($parser, XML_OPTION_CASE_FOLDING, false);

// vytvoření instancí tříd s obslužnými metodami
$mainHandler = new MainHandler($parser);
$odberatelHandler = new OdberatelHandler($parser);

// vzájemné zaregistrování tříd pro obsluhu událostí
$mainHandler->setOdberatelHandler($odberatelHandler);
$odberatelHandler->setMainHandler($mainHandler);

// nastavení objektu, ve kterém se funkce pro obsluhu událostí nacházejí jako metody
xml_set_object($parser, $mainHandler);

// nastavení metod pro obsluhu událostí
xml_set_element_handler($parser, "startElement", "endElement");
xml_set_character_data_handler($parser, "characters");

// otevření XML dokumentu s fakturou
$fp = fopen($filename, "r");
if (!$fp) die ("Nelze otevřít soubor: $filename.");

// zpracování celého souboru
while ($data = fread($fp, 4096))
{
  if (!xml_parse($parser, $data, feof($fp)))
    die (sprintf("\nChyba při čtení XML: '%s' na řádce %d sloupec %d (offset %d).\n",
                 xml_error_string(xml_get_error_code($parser)),
                 xml_get_current_line_number($parser),
                 xml_get_current_column_number($parser),
                 xml_get_current_byte_index($parser)));
}

// uvolnění paměti alokované parserem
xml_parser_free($parser);

// třída sdružující obsluhu událostí
class MainHandler
{
  var $parser, $odberatelHandler;
  
  // konstruktor si zapamatuje parser
  function MainHandler($parser)
  {
    $this->parser = $parser;
  }

  // nastavení třídy, která se stará o obsluhu elementu odberatel
  function setOdberatelHandler($odberatelHandler)
  {
    $this->odberatelHandler = $odberatelHandler;
  }
  
  // obsluha začátku elementu
  function startElement($parser, $name, $attrs)
  {
    // element odberatel má vlastní obsluhu událostí
    if ($name == "odberatel")
    {
      // přepnutí na jiný objekt s obsluhou událostí
      xml_set_object($this->parser, $this->odberatelHandler);
    }
  }
  
  // zpracování konce elementu
  function endElement($parser, $name)
  {
    // konce elementů nyní ignorujeme
  }
  
  // obsluha znakových dat
  function characters($parser, $data)
  {
    // textový obsah ignorujeme
  }
}

// třída sdružující obsluhu událostí pro element odberatel
class OdberatelHandler
{
  var $parser, $mainHandler;
  
  // konstruktor si zapamatuje parser
  function OdberatelHandler($parser)
  {
    $this->parser = $parser;
  }
  
  // nastavení třídy pro obsluhu dokumentu mimo element odberatel
  function setMainHandler($mainHandler)
  {
    $this->mainHandler = $mainHandler;
  }
  
  // obsluha začátku elementu
  function startElement($parser, $name, $attrs)
  {
    if ($name == "nazev") echo "Název: ";
    if ($name == "adresa") echo "Adresa: ";
    if ($name == "ico") echo "IČ: ";
    if ($name == "dic") echo "DIČ: ";
  }
  
  // zpracování konce elementu
  function endElement($parser, $name)
  {
    // na konci elementu odberatel musíme přepnout obsluhu událostí zpět
    if ($name == "odberatel")
    {
      // přepnutí na původní obsluhu událostí
      xml_set_object($this->parser, $this->mainHandler);
    }
    else
    {
      // za elementy se dělá konec řádku
      echo "<br>\n";
    }
  }
  
  // obsluha znakových dat
  function characters($parser, $data)
  {
    echo $data;
  }
}

?>